@@ -1,23 +1,37 @@
+# Copyright  2014-2021 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
 import re
-from typing import Any, Optional, TypeVar, Type, Dict
+from ipaddress import ip_address
+from typing import Any, Dict, Optional, Tuple, Type, TypeVar
 
-from aiohttp import ClientSession
+from duniterpy import constants as const
 
-import duniterpy.constants as constants
 from ..documents import MalformedDocumentError
 
 
 class ConnectionHandler:
-    """Helper class used by other API classes to ease passing server connection information."""
+    """Helper class used by other API classes to ease passing address connection information."""
 
     def __init__(
         self,
         http_scheme: str,
         ws_scheme: str,
-        server: str,
+        address: str,
         port: int,
         path: str,
-        session: ClientSession,
         proxy: Optional[str] = None,
     ) -> None:
         """
@@ -25,22 +39,20 @@ class ConnectionHandler:
 
         :param http_scheme: Http scheme
         :param ws_scheme: Web socket scheme
-        :param server: Server IP or domain name
+        :param address: Domain name, IPv6, or IPv4 address
         :param port: Port number
         :param port: Url path
-        :param session: Session AIOHTTP
         :param proxy: Proxy (optional, default=None)
         """
         self.http_scheme = http_scheme
         self.ws_scheme = ws_scheme
-        self.server = server
+        self.address = address
         self.port = port
         self.path = path
         self.proxy = proxy
-        self.session = session
 
     def __str__(self) -> str:
-        return "connection info: %s:%d" % (self.server, self.port)
+        return "connection info: %s:%d" % (self.address, self.port)
 
 
 # required to type hint cls in classmethod
@@ -55,9 +67,7 @@ class Endpoint:
     def inline(self) -> str:
         raise NotImplementedError("inline() is not implemented")
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         raise NotImplementedError("conn_handler is not implemented")
 
     def __str__(self) -> str:
@@ -91,7 +101,7 @@ class UnknownEndpoint(Endpoint):
             properties = inline.split()[1:]
             return cls(api, properties)
         except IndexError:
-            raise MalformedDocumentError(inline)
+            raise MalformedDocumentError(inline) from IndexError
 
     def inline(self) -> str:
         """
@@ -104,17 +114,14 @@ class UnknownEndpoint(Endpoint):
             doc += " {0}".format(p)
         return doc
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
-        Return connection handler from session
+        Return connection handler
 
-        :param session: AIOHTTP Session
-        :param proxy: Proxy server
+        :param proxy: Proxy address
         :return:
         """
-        return ConnectionHandler("", "", "", 0, "", ClientSession())
+        return ConnectionHandler("", "", "", 0, "")
 
     def __str__(self) -> str:
         return "{0} {1}".format(
@@ -137,23 +144,19 @@ BMAEndpointType = TypeVar("BMAEndpointType", bound="BMAEndpoint")
 class BMAEndpoint(Endpoint):
     API = "BASIC_MERKLED_API"
     re_inline = re.compile(
-        "^BASIC_MERKLED_API(?: ({host_regex}))?(?: ({ipv4_regex}))?(?: ({ipv6_regex}))?(?: ([0-9]+))$".format(
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-        )
+        f"^{API}(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))?(?: (?P<port>{const.PORT_REGEX}))$"
     )
 
-    def __init__(self, server: str, ipv4: str, ipv6: str, port: int) -> None:
+    def __init__(self, host: str, ipv4: str, ipv6: str, port: int) -> None:
         """
         Init BMAEndpoint instance
 
-        :param server: IP or domain name
+        :param host: Hostname
         :param ipv4: IP as IPv4 format
         :param ipv6: IP as IPv6 format
         :param port: Port number
         """
-        self.server = server
+        self.host = host
         self.ipv4 = ipv4
         self.ipv6 = ipv6
         self.port = port
@@ -169,11 +172,11 @@ class BMAEndpoint(Endpoint):
         m = BMAEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(BMAEndpoint.API)
-        server = m.group(1)
-        ipv4 = m.group(2)
-        ipv6 = m.group(3)
-        port = int(m.group(4))
-        return cls(server, ipv4, ipv6, port)
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+
+        return cls(host, ipv4, ipv6, port)
 
     def inline(self) -> str:
         """
@@ -182,33 +185,30 @@ class BMAEndpoint(Endpoint):
         :return:
         """
         return BMAEndpoint.API + "{DNS}{IPv4}{IPv6}{PORT}".format(
-            DNS=(" {0}".format(self.server) if self.server else ""),
+            DNS=(" {0}".format(self.host) if self.host else ""),
             IPv4=(" {0}".format(self.ipv4) if self.ipv4 else ""),
             IPv6=(" {0}".format(self.ipv6) if self.ipv6 else ""),
             PORT=(" {0}".format(self.port) if self.port else ""),
         )
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        if self.server:
+        if self.host:
             conn_handler = ConnectionHandler(
-                "http", "ws", self.server, self.port, "", session, proxy
+                "http", "ws", self.host, self.port, "", proxy
             )
         elif self.ipv6:
             conn_handler = ConnectionHandler(
-                "http", "ws", "[{0}]".format(self.ipv6), self.port, "", session, proxy
+                "http", "ws", "[{0}]".format(self.ipv6), self.port, "", proxy
             )
         else:
             conn_handler = ConnectionHandler(
-                "http", "ws", self.ipv4, self.port, "", session, proxy
+                "http", "ws", self.ipv4, self.port, "", proxy
             )
 
         return conn_handler
@@ -220,14 +220,14 @@ class BMAEndpoint(Endpoint):
         if not isinstance(other, BMAEndpoint):
             return NotImplemented
         return (
-            self.server == other.server
+            self.host == other.host
             and self.ipv4 == other.ipv4
             and self.ipv6 == other.ipv6
             and self.port == other.port
         )
 
     def __hash__(self) -> int:
-        return hash((self.server, self.ipv4, self.ipv6, self.port))
+        return hash((self.host, self.ipv4, self.ipv6, self.port))
 
 
 # required to type hint cls in classmethod
@@ -237,25 +237,20 @@ SecuredBMAEndpointType = TypeVar("SecuredBMAEndpointType", bound="SecuredBMAEndp
 class SecuredBMAEndpoint(BMAEndpoint):
     API = "BMAS"
     re_inline = re.compile(
-        "^BMAS(?: ({host_regex}))?(?: ({ipv4_regex}))?(?: ({ipv6_regex}))? ([0-9]+)(?: ({path_regex}))?$".format(
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-            path_regex=constants.PATH_REGEX,
-        )
+        f"^{API}(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))? (?P<port>{const.PORT_REGEX})(?: (?P<path>{const.PATH_REGEX}))?$"
     )
 
-    def __init__(self, server: str, ipv4: str, ipv6: str, port: int, path: str) -> None:
+    def __init__(self, host: str, ipv4: str, ipv6: str, port: int, path: str) -> None:
         """
         Init SecuredBMAEndpoint instance
 
-        :param server: IP or domain name
+        :param host: Hostname
         :param ipv4: IP as IPv4 format
         :param ipv6: IP as IPv6 format
         :param port: Port number
         :param path: Url path
         """
-        super().__init__(server, ipv4, ipv6, port)
+        super().__init__(host, ipv4, ipv6, port)
         self.path = path
 
     @classmethod
@@ -271,14 +266,14 @@ class SecuredBMAEndpoint(BMAEndpoint):
         m = SecuredBMAEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(SecuredBMAEndpoint.API)
-        server = m.group(1)
-        ipv4 = m.group(2)
-        ipv6 = m.group(3)
-        port = int(m.group(4))
-        path = m.group(5)
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+        path = m["path"]
+
         if not path:
             path = ""
-        return cls(server, ipv4, ipv6, port, path)
+        return cls(host, ipv4, ipv6, port, path)
 
     def inline(self) -> str:
         """
@@ -288,38 +283,29 @@ class SecuredBMAEndpoint(BMAEndpoint):
         """
         inlined = [
             str(info)
-            for info in (self.server, self.ipv4, self.ipv6, self.port, self.path)
+            for info in (self.host, self.ipv4, self.ipv6, self.port, self.path)
             if info
         ]
         return SecuredBMAEndpoint.API + " " + " ".join(inlined)
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        if self.server:
+        if self.host:
             conn_handler = ConnectionHandler(
-                "https", "wss", self.server, self.port, self.path, session, proxy
+                "https", "wss", self.host, self.port, self.path, proxy
             )
         elif self.ipv6:
             conn_handler = ConnectionHandler(
-                "https",
-                "wss",
-                "[{0}]".format(self.ipv6),
-                self.port,
-                self.path,
-                session,
-                proxy,
+                "https", "wss", "[{0}]".format(self.ipv6), self.port, self.path, proxy
             )
         else:
             conn_handler = ConnectionHandler(
-                "https", "wss", self.ipv4, self.port, self.path, session, proxy
+                "https", "wss", self.ipv4, self.port, self.path, proxy
             )
 
         return conn_handler
@@ -332,18 +318,12 @@ WS2PEndpointType = TypeVar("WS2PEndpointType", bound="WS2PEndpoint")
 class WS2PEndpoint(Endpoint):
     API = "WS2P"
     re_inline = re.compile(
-        "^WS2P ({ws2pid_regex}) ((?:{host_regex})|(?:{ipv4_regex})|(?:{ipv6_regex})) ([0-9]+)?(?: ({path_regex}))?$".format(
-            ws2pid_regex=constants.WS2PID_REGEX,
-            host_regex=constants.HOST_REGEX,
-            ipv4_regex=constants.IPV4_REGEX,
-            ipv6_regex=constants.IPV6_REGEX,
-            path_regex=constants.PATH_REGEX,
-        )
+        f"^{API} (?P<ws2pid>{const.WS2PID_REGEX}) (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})|(?:{const.IPV6_REGEX})) (?P<port>{const.PORT_REGEX})?(?: (?P<path>{const.PATH_REGEX}))?$"
     )
 
-    def __init__(self, ws2pid: str, server: str, port: int, path: str) -> None:
+    def __init__(self, ws2pid: str, host: str, port: int, path: str) -> None:
         self.ws2pid = ws2pid
-        self.server = server
+        self.host = host
         self.port = port
         self.path = path
 
@@ -358,13 +338,13 @@ class WS2PEndpoint(Endpoint):
         m = WS2PEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(WS2PEndpoint.API)
-        ws2pid = m.group(1)
-        server = m.group(2)
-        port = int(m.group(3))
-        path = m.group(4)
+        ws2pid = m["ws2pid"]
+        host = m["host"]
+        port = int(m["port"])
+        path = m["path"]
         if not path:
             path = ""
-        return cls(ws2pid, server, port, path)
+        return cls(ws2pid, host, port, path)
 
     def inline(self) -> str:
         """
@@ -373,19 +353,14 @@ class WS2PEndpoint(Endpoint):
         :return:
         """
         inlined = [
-            str(info)
-            for info in (self.ws2pid, self.server, self.port, self.path)
-            if info
+            str(info) for info in (self.ws2pid, self.host, self.port, self.path) if info
         ]
         return WS2PEndpoint.API + " " + " ".join(inlined)
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
@@ -395,13 +370,7 @@ class WS2PEndpoint(Endpoint):
             http_scheme += "s"
             websocket_scheme += "s"
         return ConnectionHandler(
-            http_scheme,
-            websocket_scheme,
-            self.server,
-            self.port,
-            self.path,
-            session,
-            proxy,
+            http_scheme, websocket_scheme, self.host, self.port, self.path, proxy
         )
 
     def __str__(self) -> str:
@@ -411,14 +380,14 @@ class WS2PEndpoint(Endpoint):
         if not isinstance(other, WS2PEndpoint):
             return NotImplemented
         return (
-            self.server == other.server
+            self.host == other.host
             and self.ws2pid == other.ws2pid
             and self.port == other.port
             and self.path == other.path
         )
 
     def __hash__(self) -> int:
-        return hash((self.ws2pid, self.server, self.port, self.path))
+        return hash((self.ws2pid, self.host, self.port, self.path))
 
 
 # required to type hint cls in classmethod
@@ -428,13 +397,11 @@ ESCoreEndpointType = TypeVar("ESCoreEndpointType", bound="ESCoreEndpoint")
 class ESCoreEndpoint(Endpoint):
     API = "ES_CORE_API"
     re_inline = re.compile(
-        "^ES_CORE_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -448,9 +415,9 @@ class ESCoreEndpoint(Endpoint):
         m = ESCoreEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESCoreEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -458,22 +425,17 @@ class ESCoreEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
+        inlined = [str(info) for info in (self.host, self.port) if info]
         return ESCoreEndpoint.API + " " + " ".join(inlined)
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -481,10 +443,10 @@ class ESCoreEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESCoreEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((self.server, self.port))
+        return hash((self.host, self.port))
 
 
 # required to type hint cls in classmethod
@@ -494,13 +456,11 @@ ESUserEndpointType = TypeVar("ESUserEndpointType", bound="ESUserEndpoint")
 class ESUserEndpoint(Endpoint):
     API = "ES_USER_API"
     re_inline = re.compile(
-        "^ES_USER_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -514,9 +474,9 @@ class ESUserEndpoint(Endpoint):
         m = ESUserEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESUserEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -524,22 +484,17 @@ class ESUserEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
+        inlined = [str(info) for info in (self.host, self.port) if info]
         return ESUserEndpoint.API + " " + " ".join(inlined)
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -547,10 +502,10 @@ class ESUserEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESUserEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((self.server, self.port))
+        return hash((self.host, self.port))
 
 
 # required to type hint cls in classmethod
@@ -562,13 +517,11 @@ ESSubscribtionEndpointType = TypeVar(
 class ESSubscribtionEndpoint(Endpoint):
     API = "ES_SUBSCRIPTION_API"
     re_inline = re.compile(
-        "^ES_SUBSCRIPTION_API ((?:{host_regex})|(?:{ipv4_regex})) ([0-9]+)$".format(
-            host_regex=constants.HOST_REGEX, ipv4_regex=constants.IPV4_REGEX
-        )
+        f"^{API} (?P<host>(?:{const.HOST_REGEX})|(?:{const.IPV4_REGEX})) (?P<port>{const.PORT_REGEX})$"
     )
 
-    def __init__(self, server: str, port: int) -> None:
-        self.server = server
+    def __init__(self, host: str, port: int) -> None:
+        self.host = host
         self.port = port
 
     @classmethod
@@ -584,9 +537,9 @@ class ESSubscribtionEndpoint(Endpoint):
         m = ESSubscribtionEndpoint.re_inline.match(inline)
         if m is None:
             raise MalformedDocumentError(ESSubscribtionEndpoint.API)
-        server = m.group(1)
-        port = int(m.group(2))
-        return cls(server, port)
+        host = m["host"]
+        port = int(m["port"])
+        return cls(host, port)
 
     def inline(self) -> str:
         """
@@ -594,22 +547,17 @@ class ESSubscribtionEndpoint(Endpoint):
 
         :return:
         """
-        inlined = [str(info) for info in (self.server, self.port) if info]
+        inlined = [str(info) for info in (self.host, self.port) if info]
         return ESSubscribtionEndpoint.API + " " + " ".join(inlined)
 
-    def conn_handler(
-        self, session: ClientSession, proxy: str = None
-    ) -> ConnectionHandler:
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
         """
         Return connection handler instance for the endpoint
 
-        :param session: AIOHTTP client session instance
         :param proxy: Proxy url
         :return:
         """
-        return ConnectionHandler(
-            "https", "wss", self.server, self.port, "", session, proxy
-        )
+        return ConnectionHandler("https", "wss", self.host, self.port, "", proxy)
 
     def __str__(self) -> str:
         return self.inline()
@@ -617,10 +565,137 @@ class ESSubscribtionEndpoint(Endpoint):
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, ESSubscribtionEndpoint):
             return NotImplemented
-        return self.server == other.server and self.port == other.port
+        return self.host == other.host and self.port == other.port
 
     def __hash__(self) -> int:
-        return hash((ESSubscribtionEndpoint.API, self.server, self.port))
+        return hash((ESSubscribtionEndpoint.API, self.host, self.port))
+
+
+# required to type hint cls in classmethod
+GVAEndpointType = TypeVar("GVAEndpointType", bound="GVAEndpoint")
+
+
+class GVAEndpoint(Endpoint):
+    API = "GVA"
+    endpoint_format = f"^GVA(?: (?P<flags>{const.ENDPOINT_FLAGS_REGEX}))?(?: (?P<host>{const.HOST_REGEX}))?(?: (?P<ipv4>{const.IPV4_REGEX}))?(?: (?P<ipv6>{const.IPV6_REGEX}))? (?P<port>{const.PORT_REGEX})(?: (?P<path>{const.PATH_REGEX}))?$"
+    re_inline = re.compile(endpoint_format)
+
+    def __init__(
+        self,
+        flags: str,
+        host: str,
+        ipv4: str,
+        ipv6: str,
+        port: int,
+        path: str,
+    ) -> None:
+        """
+        Init GVAEndpoint instance
+
+        :param flags: Flags of endpoint
+        :param host: Hostname
+        :param ipv4: IP as IPv4 format
+        :param ipv6: IP as IPv6 format
+        :param port: Port number
+        :param path: Url path
+        """
+        self.flags = flags
+        self.host = host
+        self.ipv4 = ipv4
+        self.ipv6 = ipv6
+        self.port = port
+        self.path = path
+
+    @classmethod
+    def from_inline(cls: Type[GVAEndpointType], inline: str) -> GVAEndpointType:
+        """
+        Return GVAEndpoint instance from endpoint string
+
+        :param inline: Endpoint string
+        :return:
+        """
+        m = cls.re_inline.match(inline)
+        if m is None:
+            raise MalformedDocumentError(cls.API)
+        flags = m["flags"]
+        host, ipv4 = fix_host_ipv4_mix_up(m["host"], m["ipv4"])
+        ipv6 = m["ipv6"]
+        port = int(m["port"])
+        path = m["path"]
+
+        if not flags:
+            flags = ""
+        if not path:
+            path = ""
+        return cls(flags, host, ipv4, ipv6, port, path)
+
+    def inline(self) -> str:
+        """
+        Return endpoint string
+
+        :return:
+        """
+        inlined = [
+            str(info)
+            for info in (
+                self.flags,
+                self.host,
+                self.ipv4,
+                self.ipv6,
+                self.port,
+                self.path,
+            )
+            if info
+        ]
+        return self.API + " " + " ".join(inlined)
+
+    def conn_handler(self, proxy: str = None) -> ConnectionHandler:
+        """
+        Return connection handler instance for the endpoint
+
+        :param proxy: Proxy url
+        :return:
+        """
+        scheme_http = "https" if "S" in self.flags else "http"
+        scheme_ws = "wss" if "S" in self.flags else "ws"
+
+        if self.host:
+            conn_handler = ConnectionHandler(
+                scheme_http, scheme_ws, self.host, self.port, self.path, proxy
+            )
+        elif self.ipv6:
+            conn_handler = ConnectionHandler(
+                scheme_http,
+                scheme_ws,
+                "[{0}]".format(self.ipv6),
+                self.port,
+                self.path,
+                proxy,
+            )
+        else:
+            conn_handler = ConnectionHandler(
+                scheme_http, scheme_ws, self.ipv4, self.port, self.path, proxy
+            )
+
+        return conn_handler
+
+    def __str__(self) -> str:
+        return self.inline()
+
+    def __eq__(self, other: Any) -> bool:
+        if not isinstance(other, self.__class__):
+            return NotImplemented
+        return (
+            self.flags == other.flags
+            and self.host == other.host
+            and self.ipv4 == other.ipv4
+            and self.ipv6 == other.ipv6
+            and self.port == other.port
+            and self.path == other.path
+        )
+
+    def __hash__(self) -> int:
+        return hash((self.flags, self.host, self.ipv4, self.ipv6, self.port, self.path))
 
 
 MANAGED_API = {
@@ -630,6 +705,7 @@ MANAGED_API = {
     ESCoreEndpoint.API: ESCoreEndpoint,
     ESUserEndpoint.API: ESUserEndpoint,
     ESSubscribtionEndpoint.API: ESSubscribtionEndpoint,
+    GVAEndpoint.API: GVAEndpoint,
 }  # type: Dict[str, Any]
 
 
@@ -654,3 +730,12 @@ def endpoint(value: Any) -> Any:
         raise TypeError("Cannot convert {0} to endpoint".format(value))
 
     return result
+
+
+def fix_host_ipv4_mix_up(host: str, ipv4: str) -> Tuple[str, str]:
+    mixed_up = False
+    try:
+        mixed_up = ip_address(host).version == 4 and not ipv4
+    except ValueError:
+        pass
+    return ("", host) if mixed_up else (host, ipv4)
