@@ -1,8 +1,23 @@
-from typing import Optional, TypeVar, Type, Any, Union
+# Copyright  2014-2021 Vincent Texier <vit@free.fr>
+#
+# DuniterPy is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 3 of the License, or
+# (at your option) any later version.
+#
+# DuniterPy is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from pypeg2 import re, attr, Keyword, Enum, contiguous, maybe_some, whitespace, K
+from typing import Any, Optional, Type, TypeVar, Union
 
-from ..constants import PUBKEY_REGEX, HASH_REGEX
+from pypeg2 import Enum, K, Keyword, attr, contiguous, maybe_some, re, whitespace
+
+from ..constants import HASH_REGEX, PUBKEY_REGEX
 
 
 class Pubkey(str):
